use super::item::Item;
use ansi_markup::parse_ansi_markup;
use serde::{Deserialize, Serialize};

#[derive(PartialEq, Eq, PartialOrd, Clone, Serialize, Deserialize, Debug, Ord)]
pub enum EffectType {
    Ephemeral, // i.e. no enduring impact to show in status.
    Bandages,
    Bleed,
    Stunned,
    CurrentRoom,
}

pub struct EffectSet {
    pub effect_type: EffectType,
    pub effects: Vec<Effect>,
}

pub enum Effect {
    // messagef takes player, causative item, target as the 3 parameters. Returns message.
    BroadcastMessage {
        delay_secs: u64,
        messagef: Box<dyn Fn(&Item, &Item, &Item) -> String + Sync + Send>,
    },
    DirectMessage {
        delay_secs: u64,
        messagef: Box<dyn Fn(&Item, &Item, &Item) -> String + Sync + Send>,
    },
    // skill_multiplier is always positive - sign flipped for crit fails.
    ChangeTargetHealth {
        delay_secs: u64,
        base_effect: i64,
        skill_multiplier: f64,
        max_effect: i64,
        message: Box<dyn Fn(&Item) -> String + Sync + Send>,
    },
}

#[derive(Serialize, Deserialize)]
pub enum SimpleEffect {
    BroadcastMessage {
        delay_secs: u64,
        message: String,
    },
    DirectMessage {
        delay_secs: u64,
        message: String,
    },
    // skill_multiplier is always positive - sign flipped for crit fails.
    ChangeTargetHealth {
        delay_secs: u64,
        base_effect: i64,
        skill_multiplier: f64,
        max_effect: i64,
        message: String,
    },
}

impl From<&SimpleEffect> for Effect {
    fn from(simple: &SimpleEffect) -> Effect {
        match simple {
            SimpleEffect::BroadcastMessage {
                delay_secs,
                message,
            } => {
                let messagem = parse_ansi_markup(message).unwrap() + "\n";
                Effect::BroadcastMessage {
                    delay_secs: *delay_secs,
                    messagef: Box::new(move |_, _, _| messagem.clone()),
                }
            }
            SimpleEffect::DirectMessage {
                delay_secs,
                message,
            } => {
                let messagem = parse_ansi_markup(message).unwrap() + "\n";
                Effect::DirectMessage {
                    delay_secs: *delay_secs,
                    messagef: Box::new(move |_, _, _| messagem.clone()),
                }
            }
            SimpleEffect::ChangeTargetHealth {
                delay_secs,
                base_effect,
                skill_multiplier,
                max_effect,
                message,
            } => {
                let messagem = parse_ansi_markup(message).unwrap() + "\n";
                Effect::ChangeTargetHealth {
                    delay_secs: *delay_secs,
                    base_effect: *base_effect,
                    skill_multiplier: *skill_multiplier,
                    max_effect: *max_effect,
                    message: Box::new(move |_| messagem.clone()),
                }
            }
        }
    }
}
