use super::{NPCPronounType, NPCSayInfo, NPCSayType, NPCSpawnPossession, NPC};
use crate::{
    models::{
        consent::ConsentType,
        item::{LocationActionType, Pronouns, SkillType},
    },
    static_content::{
        npc::{npc_pronoun_type_to_pronouns, KillBonus},
        possession_type::PossessionType,
        species::SpeciesType,
    },
};
use serde::Deserialize;
use serde_yaml::from_str as from_yaml_str;

#[derive(Deserialize)]
enum SewerNPC {
    RadiantPredator {
        code: String,
        name: String,
        spawn_loc: String,
        gender: NPCPronounType,
    },
    KillerCroc {
        code: String,
        name: String,
        spawn_loc: String,
    },
    Stinkfiend {
        code: String,
        name: String,
        spawn_loc: String,
    },
}

pub fn npc_list() -> Vec<NPC> {
    use NPCSayType::FromFixedList;
    let radiant_predator_stdsay = NPCSayInfo {
        say_code: "babble",
        frequency_secs: 30,
        talk_type: FromFixedList(vec![
            "I hunger for flesh.",
            "The darkness is my ally.",
            "Step closer please, prey.",
            "My stomach rumbles for human flesh.",
            "Survival is a predator's only law.",
            "The scent of fear is intoxicating.",
            "I am the shadow in the sewer.",
            "In the dark, I thrive.",
            "Flesh is my sustenance.",
            "My predator knife is fantastic for hunting unsuspecting prey.",
            "In the depths, I reign supreme.",
            "The hunt begins.",
            "No mercy for the intruders.",
            "I love visitors to the sewers, they taste great!",
            "The echoes reveal your presence.",
            "My hunger knows no bounds.",
            "The predator way is to be silent, deadly, and relentless.",
            "The sewer is my kingdom.",
            "Prey that comes my way doesn't get away.",
            "Do I smell a human... in my sewer? Yum.",
            "I know you are there, prey! Give up, we might as well get this over with!",
        ]),
    };
    let stinkfiend_stdsay = NPCSayInfo {
        say_code: "babble",
        frequency_secs: 30,
        talk_type: FromFixedList(vec![
            "This part of the sewers is mine, and all who enter die by my hand!",
            "I hate surface dwellers so much. At least they're easy to kill!",
            "Crocodiles don't bother me - I just whip their faces off.",
            "A surface dweller once said I stink. I think I still have his skull.",
            "Surface dwellers are good for stress relief - whipping them is fun!",
            "They come from the surface and think they can just waltz in. Idiots.",
            "I think the surface makes people weak.",
            "Once I killed a surface dweller with the body of his friend.",
            "Us stinkfiends are genetically superior, and one day we will wipe out all others.",
            "I dream of a day when stinkfiends rule, and the inferior surface dwellers are wiped out.",
            "Death to surface dwellers!",
            "There is one surface dweller who hides in a sub-cellar around here who I have never been able to catch. It makes my blood boil. One day I will get her!",
        ]),
    };

    from_yaml_str::<Vec<SewerNPC>>(include_str!("sewer_npcs.yaml"))
        .unwrap()
        .into_iter()
        .map(|c| match c {
            SewerNPC::RadiantPredator { code, name, spawn_loc, gender } =>
              NPC {
                  code: format!("sewer_{}", &code),
                  name,
                  pronouns: Pronouns {
                      is_proper: false,
                      ..npc_pronoun_type_to_pronouns(&gender)
                  },
                  aliases: vec!["predator".to_owned(), "radiant".to_owned(),
                                "radiant predator".to_owned()],
                  aggression: 12,
                  description: "A creature, vicious in demeanour, but uncannily human like, but also a bit... off, in a grotesquely mutant way. Discoloured flesh hangs loosely around the disproportionate frame, while a horrific grin reveals phosphorescent yellowish teeth, sharper than those of a normal human. You catch a whiff of a putrid odour, and notice fragments of meat caught between the teeth of this hideous humanoid. Coarse brown matted hair over the body gives it an ape-like appearance. The eyes are larger than those of a normal human and gleam, leading you to suspect this creature is well adapted to low light levels".to_owned(),
                  spawn_location: format!("room/melbs_sewers_{}", &spawn_loc),
                  spawn_possessions: vec![
                      NPCSpawnPossession {
                          what: PossessionType::RadiantPredatorDagger,
                          action_type: LocationActionType::Wielded,
                          wear_layer: 0,
                      },
                  ],
                  kill_bonus: Some(KillBonus {
                      msg: "On your wristpad: You are doing a great job removing hazards from the sewer! Let me toss a few credits your way.",
                      payment: 200,
                  }),
                  aggro_pc_only: true,
                  total_xp: 10000,
                  total_skills: SkillType::values()
                      .into_iter()
                      .map(|sk| {
                          (
                              sk.clone(),
                              match sk {
                                  SkillType::Dodge => 14.0,
                                  SkillType::Blades => 18.0,
                                  _ => 8.0
                              }
                          )
                      }).collect(),
                  player_consents: vec!(ConsentType::Fight),
                  message_handler: None,
                  wander_zones: vec!["melbs_sewers".to_owned()],
                  says: vec!(radiant_predator_stdsay.clone()),
                  ..Default::default()
              },
            SewerNPC::KillerCroc { code, name, spawn_loc } =>
                NPC {
                  code: format!("sewer_croc_{}", &code),
                  name,
                  pronouns: Pronouns {
                      is_proper: false,
                      ..Pronouns::default_inanimate()
                  },
                  aliases: vec!["crocodile".to_owned()],
                  aggression: 12,
                  description: "A saltwater crocodile, featuring rows of extremely sharp white teeth gleaming in the light, and tough olive-green scales that will clearly protect it from a lot of abuse. It is longer than a person, and looks extremely menacing".to_owned(),
                  spawn_location: format!("room/melbs_sewers_{}", &spawn_loc),
                  species: SpeciesType::Crocodile,
                  kill_bonus: Some(KillBonus {
                      msg: "On your wristpad: I can't believe you took down a salty! Here's something for your trouble.",
                      payment: 300,
                  }),
                  max_health: 60,
                  aggro_pc_only: true,
                  total_xp: 20000,
                  total_skills: SkillType::values()
                      .into_iter()
                      .map(|sk| {
                          (
                              sk.clone(),
                              match sk {
                                  SkillType::Dodge => 14.0,
                                  SkillType::Fists => 20.0,
                                  _ => 8.0
                              }
                          )
                      }).collect(),
                  player_consents: vec!(ConsentType::Fight),
                  intrinsic_weapon: Some(PossessionType::Fangs),
                  message_handler: None,
                  wander_zones: vec!["melbs_sewers".to_owned()],
                  ..Default::default()
                },
            SewerNPC::Stinkfiend { code, name, spawn_loc } =>
                NPC {
                  code: format!("stinkfiend_{}", &code),
                  name,
                  pronouns: Pronouns {
                      is_proper: false,
                      ..Pronouns::default_inanimate()
                  },
                  aliases: vec!["fiend".to_owned()],
                  aggression: 21,
                  description: "A grotesquely mutant humanoid creature, with orange-tinged discoloured skin hanging loose on flesh, and distorted yet horrific features. It looks dangerous. The odour of this creature is so pungent you have to choke back the urge to vomit as it wafts your way".to_owned(),
                  spawn_location: format!("room/melbs_sewers_{}", &spawn_loc),
                  species: SpeciesType::Human,
                  kill_bonus: Some(KillBonus {
                      msg: "On your wristpad: You legend - you killed a bloody stinkfiend! You're braver than I am mate - that deserves a reward!",
                      payment: 500,
                  }),
                  max_health: 70,
                  aggro_pc_only: true,
                  total_xp: 30000,
                  total_skills: SkillType::values()
                      .into_iter()
                      .map(|sk| {
                          (
                              sk.clone(),
                              match sk {
                                  SkillType::Dodge => 18.0,
                                  SkillType::Whips => 20.0,
                                  _ => 8.0
                              }
                          )
                      }).collect(),
                  player_consents: vec!(ConsentType::Fight),
                  spawn_possessions: vec![
                      NPCSpawnPossession {
                          what: PossessionType::Sjambok,
                          action_type: LocationActionType::Wielded,
                          wear_layer: 0,
                      },
                  ],
                  says: vec!(stinkfiend_stdsay.clone()),
                  message_handler: None,
                  wander_zones: vec![], // They defend their room.
                  ..Default::default()
                }
        }).collect()
}
